'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'Copyright 2002 Seapine Software, Inc. All rights reserved.

'IMPORTANT:  This Software is supplied to you by Seapine Software, Inc. (Seapine)
'in consideration of your agreement to the following terms, and your use, installation,
'modification or redistribution of this Software constitutes acceptance of these terms.
'If you do not agree with these terms, please do not use, install, modify or redistribute
'this Software.

'In consideration of your agreement to abide by the following terms, and subject to these
'terms, Seapine grants you a personal, non-exclusive license, under Seapine copyrights in
'this original Seapine software (the Software), to use, reproduce, modify and redistribute
'the Software, with or without modifications, in source and/or binary forms; provided that
'if you redistribute the Software in its entirety and without modifications, you must retain
'this notice and the following text and disclaimers in all such redistributions of the Software.
'Neither the name, trademarks, service marks or logos of Seapine, Inc. may be used to endorse
'or promote products derived from the Software without specific prior written permission from
'Seapine.  Except as expressly stated in this notice, no other rights or licenses, express or
'implied, are granted by Seapine herein, including but not limited to any patent rights that
'may be infringed by your derivative works or by other works in which the Software may be
'incorporated.

'The Software is provided by Seapine on an "AS IS" basis.  SEAPINE MAKES NO WARRANTIES,
'EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT,
'MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE SOFTWARE OR ITS USE AND
'OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS. 

'IN NO EVENT SHALL SEAPINE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL
'DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
'LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE,
'REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE SOFTWARE, HOWEVER CAUSED AND WHETHER
'UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN
'IF SEAPINE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Imports System
Imports System.Collections
Imports System.Reflection

Module TTCreateUser

   '''''''''''''''''''''''''''''''''''''''
   'CTTCreateUserModule TTCreateUser
   '''''
   Class CTTCreateUser
      Dim loginUser As SLoginUser         ' The user to log in as
      Dim newUser As SUser                ' The user/customer to create
      Dim cookie As Long                  ' Cookie returned after logging into a database
      Dim soapServer As ttsoapcgi         ' The TestTrack Pro SOAP Server
      Private Enum eUserType              ' Type of user to create
         User
         Customer
      End Enum

      ''''''''''''''''''''''''''''''''''''''
      'Main
      '''''
      Shared Sub Main()
         Dim createUser As New CTTCreateUser
         Dim args() As String
         args = Command().Split(" ")

         If args(0).ToString() = "" Or args(0).ToString() = "/h" Or args(0).ToString() = "-h" Then
            Call createUser.ShowUsage()
            Return
         End If

         Dim arguments As New CCommandLineWrapper
         arguments.AddSingleArgs(New String() {"-c"})
         arguments.AddDoubleArgs(New String() {"-lu", "-lp", "-d", "-f", "-l", "-un", "-ug", "-p", "-ul"})

         If arguments.ScanCommandLine(args) Then
            If createUser.ExtractDataFromArguments(arguments) Then
               createUser.CreateUser()
            Else
               createUser.ShowUsage()
            End If
         Else
            createUser.ShowUsage()
         End If
      End Sub

      '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
      ' CreateUser - Handles the details of creating a user/customer
      ''''''''''''
      Private Sub CreateUser()
         Dim UserType As String
         Dim hasPerms As Boolean
         hasPerms = False

         If Not VerifyDatabase() Then
            Console.WriteLine("The specified user: {0} is not able to login to database: {1}", _
               loginUser.userName, loginUser.database)
            Exit Sub
         End If

         Try
            cookie = soapServer.DatabaseLogon(loginUser.database, loginUser.userName, loginUser.password)

            If newUser.isCustomer Then
               UserType = "customers"
               hasPerms = CanWorkWithUser(eUserType.Customer)
            Else
               UserType = "users"
               hasPerms = CanWorkWithUser(eUserType.User)
            End If

            If hasPerms Then
               If Not newUser.isCustomer And newUser.userName = String.Empty Then
                  Console.WriteLine("Users must have a Username and Group.")
                  Exit Sub
               End If

               Dim user As New CUser

               ' Note: The following three options must be set to true if you want the
               ' user/customer to be able to login to TestTrack Pro
               If newUser.userName = String.Empty Then
                  user.isglobal = False
                  user.isactiveglobal = False
               Else
                  user.isglobal = True
                  user.isactiveglobal = True
               End If

               user.isactive = True
               user.firstname = newUser.firstName
               user.lastname = newUser.lastName
               user.loginname = newUser.userName
               user.password = newUser.password
               user.usergroupname = newUser.userGroup
               user.iscustomer = newUser.isCustomer
               user.userLicense = newUser.userLicense

               If user.iscustomer Then
                  soapServer.addCustomer(cookie, user)
               Else
                  soapServer.addUser(cookie, user)
               End If

            Else
               Console.WriteLine("You do not have permission to add {0}.", UserType)
            End If

         Catch e As System.Exception
            Console.WriteLine(e.Message)

         Finally
            ' Disconnect
            If Not cookie = 0 Then
               Try
                  soapServer.DatabaseLogoff(cookie)
               Catch e As System.Exception
                  Console.WriteLine("Error while attempting to disconnect, error message is: {0}", e.Message)
               End Try
            End If
         End Try
      End Sub

      ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
      ' ShowUsage - Shows program usage
      ''''''''''
      Private Sub ShowUsage()
         Dim myModule As System.Reflection.Module
         myModule = System.Reflection.Assembly.GetExecutingAssembly().GetModules()(0)
         Dim progName As String
         progName = myModule.Name.Substring(0, myModule.Name.Length - 4).ToUpper()
         Console.WriteLine("{0}", progName)
         Console.WriteLine("   -lu username        User to log in as")
         Console.WriteLine("  [-lp password]       Password of the above user, if blank omit")
         Console.WriteLine("   -d database         Database to log into")
         Console.WriteLine("  [-c]                 Create user as customer")
         Console.WriteLine("   -f first name       First name of created user")
         Console.WriteLine("   -l last name        Last name of created user")
         Console.WriteLine("  [-un Username]       User name for created user")
         Console.WriteLine("  [-ug Usergroup]      User group for created user (must be used if -un is used)")
         Console.WriteLine("  [-p Password]        Password for new user")
         Console.WriteLine("  [-ul License Type]   License type for new user (Floating or Named)")
      End Sub

      '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
      ' ExtractDataFromArguments
      ''''''''''
      Public Function ExtractDataFromArguments(ByVal args As CCommandLineWrapper) As Boolean
         Dim result As Boolean
         result = True
         Dim lookup As Hashtable
         lookup = args.GetHashTable()

         ' Verify parameters
         If Not lookup.ContainsKey("-lu") Then
            Console.WriteLine("Error, login user not specified,")
            result = False
         ElseIf Not lookup.ContainsKey("-d") Then
            Console.WriteLine("Error, database not specified.")
            result = False
         ElseIf Not lookup.ContainsKey("-f") And Not lookup.ContainsKey("-l") Then
            Console.WriteLine("Error, first and/or last name not specified.")
            result = False
         End If

         ' Failed validation?
         If Not result Then
            ExtractDataFromArguments = result
         End If

         '''' Get data ''''
         ' Login user parameters
         loginUser.userName = Convert.ToString(lookup.Item("-lu"))
         loginUser.password = Convert.ToString(lookup.Item("-lp"))
         loginUser.database = Convert.ToString(lookup.Item("-d"))
         ' Created user parameters
         newUser.isCustomer = lookup.ContainsKey("-c")
         newUser.firstName = Convert.ToString(lookup.Item("-f"))
         newUser.lastName = Convert.ToString(lookup.Item("-l"))
         newUser.userGroup = Convert.ToString(lookup.Item("-ug"))
         newUser.userName = Convert.ToString(lookup.Item("-un"))
         newUser.password = Convert.ToString(lookup.Item("-p"))
         newUser.userLicense = Convert.ToString(lookup.Item("-ul"))

         ExtractDataFromArguments = result
      End Function

      '''''''''''''''''''''''''''''''''''''''''''''''''''''''''
      ' CTTCreateUser (constructor)
      ''''''''''''''
      Public Sub New()
         loginUser = New SLoginUser
         newUser = New SUser
         soapServer = New ttsoapcgi
      End Sub

      '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
      ' VerifyDatabase - Verifies if the database specified exists on the server
      ''''''''''''''
      Private Function VerifyDatabase() As Boolean
         Dim result As Boolean
         result = False
         Dim dbList() As CDatabase
         dbList = soapServer.getDatabaseList()

         Dim db As CDatabase
         For Each db In dbList
            If db.name = loginUser.database Then
               result = True
               Exit For
            End If
         Next db

         VerifyDatabase = result
      End Function

      '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
      ' CanWorkWithUser - Determines if you can edit users or customers based on
      '                   the type passed in
      ''''''''''''
      Private Function CanWorkWithUser(ByVal userType As eUserType) As Boolean
         Dim result As Boolean
         result = False
         Dim tableList() As CDatabaseTable
         tableList = soapServer.getTableList(cookie)

         Dim table As CDatabaseTable
         For Each table In tableList
            If table.name = "User" Or table.name = "Customer" Then
               result = True
               Exit For
            End If
         Next table

         CanWorkWithUser = result
      End Function

   End Class

End Module
