''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'Copyright 2002 Seapine Software, Inc. All rights reserved.
'
'IMPORTANT:  This Software is supplied to you by Seapine Software, Inc. (Seapine)
'in consideration of your agreement to the following terms, and your use, installation,
'modification or redistribution of this Software constitutes acceptance of these terms.
'If you do not agree with these terms, please do not use, install, modify or redistribute
'this Software.
'
'In consideration of your agreement to abide by the following terms, and subject to these
'terms, Seapine grants you a personal, non-exclusive license, under Seapine copyrights in
'this original Seapine software (the Software), to use, reproduce, modify and redistribute
'the Software, with or without modifications, in source and'or binary forms; provided that
'if you redistribute the Software in its entirety and without modifications, you must retain
'this notice and the following text and disclaimers in all such redistributions of the Software.
'Neither the name, trademarks, service marks or logos of Seapine, Inc. may be used to endorse
'or promote products derived from the Software without specific prior written permission from
'Seapine.  Except as expressly stated in this notice, no other rights or licenses, express or
'implied, are granted by Seapine herein, including but not limited to any patent rights that
'may be infringed by your derivative works or by other works in which the Software may be
'incorporated.
'
'The Software is provided by Seapine on an "AS IS" basis.  SEAPINE MAKES NO WARRANTIES,
'EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT,
'MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE SOFTWARE OR ITS USE AND
'OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS. 
'
'IN NO EVENT SHALL SEAPINE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL
'DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
'LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE,
'REPRODUCTION, MODIFICATION AND'OR DISTRIBUTION OF THE SOFTWARE, HOWEVER CAUSED AND WHETHER
'UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN
'IF SEAPINE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Imports System
Imports System.Collections

''##############################################################################
''# SLoginUser - Structure used to hold data related to the user to use to log
''#              into the database
Structure SLoginUser
   Public userName As String
   Public password As String
   Public database As String
End Structure


''##############################################################################
''# SUser - Structure used to create a new user or customer
Structure SUser
   Public isCustomer As Boolean
   Public firstName As String
   Public lastName As String
   Public userName As String
   Public password As String
   Public userGroup As String
   Public userLicense As String
End Structure


''##############################################################################
''# CCommandLineWrapper - Utility class used to wrap the command line arguments
Class CCommandLineWrapper

   '' Private member data
   Private singleArgs As ArrayList
   Private doubleArgs As ArrayList
   Public ReadOnly lookupTable As Hashtable

   Public Sub New()
      singleArgs = New ArrayList()
      doubleArgs = New ArrayList()
      lookupTable = New Hashtable()
   End Sub

   '******************************************************************************
   'AddSingleArgs - Add arguments that serve as flags or sentinels
   '''''''''''''
   Public Sub AddSingleArgs(ByVal args() As String)
      Dim key As String
      If Not args.Length = 0 Then
         For Each key In args
            If Not key = "" Then
               singleArgs.Add(key)
            End If
         Next key
      End If
   End Sub

   '******************************************************************************
   'AddDoubleArgs - Adds arguments that have values, ie it's not a 'flag' type
   '                of argument
   ''''''''''''''
   Public Sub AddDoubleArgs(ByVal args() As String)
      Dim key As String

      If Not args.Length = 0 Then
         For Each key In args
            If Not key = "" Then
               doubleArgs.Add(key)
            End If
         Next (key)
      End If
   End Sub

   '******************************************************************************
   'ScanCommandLine - Scans the given command line for arguments previously
   '                  added via AddSingleArgs and AddDoubleArgs
   ''''''''''''''''''
   Public Function ScanCommandLine(ByRef args() As String) As Boolean
      Dim result As Boolean
      result = True
      Dim i As Int32

      If Not args.Length = 0 Then
         For i = 0 To args.Length - 1 And result
            If Not singleArgs.IndexOf(args(i)) = -1 Then
               lookupTable.Add(args(i), "")
            ElseIf Not doubleArgs.IndexOf(args(i)) = -1 Then
               If (i + 1) < args.Length Then
                  lookupTable.Add(args(i), args(i + 1))
                  i = i + 1
               Else
                  Console.WriteLine("Error, missing value for argument {0}.", args(i))
                  result = False
               End If
            Else
               ' An invalid argument was specified
               Console.WriteLine("Error, ""{0}"" is an invalid argument", args(i))
               result = False
            End If
         Next i
      End If
      ScanCommandLine = result
   End Function

   '******************************************************************************
   'GetHashTable - Returns the lookup table for command line arguments
   '''''''''''
   Public Function GetHashTable() As Hashtable
      GetHashTable = lookupTable
   End Function

End Class